import * as SC from './styles'
import parsePhoneNumber from 'libphonenumber-js'
import UpdateButton from './components/UpdateButton'
import DiscardButton from './components/DiscardButton'
import StandardSelect from './components/StandardSelect'
import PhoneInput from './components/PhoneInput'
import WarningText from './components/WarningText'
import StandardInput from './components/StandardInput'
import Picture from './components/Picture'
import ProtectedInput from './components/ProtectedInput'
import DeleteAccount from './components/DeleteAccount'
import i18next from '../../lib/i18n'

import { I18nextProvider } from 'react-i18next'
import { useTranslation } from 'react-i18next'
import { FormEvent, useState, useEffect, useCallback } from 'react'
import { decreaseOpacity, validateField } from './helper'

import emailIcon from '../../assets/components/account-profile/email.svg'
import organizationIcon from '../../assets/components/account-profile/organization.svg'
import phoneIcon from '../../assets/components/account-profile/phone.svg'

import type {
  Profile,
  AccountProfileProps,
  ExtendedProfile,
  InvalidField,
  AccountProfileOptions
} from './types'

const defaultOptions: AccountProfileOptions = {
  language: 'id',
  buttonColor: 'var(--fair-green)',
  buttonColorSecondary: '',
  buttonColorTertiary: '',
  containerBoxShadow: '0 1rem 1.875rem -0.3125rem var(--transparent-green-1)',
  noPicture: false,
  mode: 'light',
  withOptData: false,
  readOnlyPhoneNumber: true,
  canRequestAccountDeletion: true,
  deleteRequested: false,
  canEditPhoneNumber: true
}

export const AccountProfile: React.FC<AccountProfileProps> = ({
  data,
  options,
  onUpdate,
  onDelete,
  onChangeNumber
}) => {
  options = { ...defaultOptions, ...options }

  const {
    t,
    i18n: { changeLanguage }
  } = useTranslation('accountProfile')

  let { buttonColorSecondary, buttonColorTertiary } = options
  const {
    language,
    buttonColor,
    containerBoxShadow,
    noPicture,
    withOptData,
    canRequestAccountDeletion
  } = options

  useEffect(() => {
    changeLanguage(language)
  }, [changeLanguage, language])

  const [profile, setProfile] = useState<Profile>({
    email: '',
    firstName: '',
    lastName: '',
    organization: '',
    phoneNumber: ''
  })

  const [newProfile, setNewProfile] = useState<ExtendedProfile>({
    ...profile,
    phoneCode: ''
  })

  const [invalidFields, setInvalidFields] = useState<InvalidField>({})
  const [edited, setEdited] = useState(false)

  const initProfile = useCallback(() => {
    setProfile(data)
    const phone = parsePhoneNumber(data.phoneNumber)
    setNewProfile({
      ...data,
      phoneNumber: phone ? `${phone.nationalNumber}` : '',
      phoneCode: phone ? `${phone.country} +${phone.countryCallingCode}` : ''
    })
  }, [data])

  useEffect(() => {
    initProfile()
  }, [initProfile])

  const isPrimeDefault =
    buttonColor === 'var(--fair-green)' || buttonColor === '#56ab2f'

  if (!buttonColorSecondary)
    buttonColorSecondary = isPrimeDefault
      ? 'var(--light-green)'
      : decreaseOpacity(buttonColor, 40)

  if (!buttonColorTertiary)
    buttonColorTertiary = isPrimeDefault
      ? 'var(--pale-green)'
      : decreaseOpacity(buttonColor, 27.5)

  options = { ...options, buttonColorSecondary, buttonColorTertiary }

  const updateEdited = useCallback(() => {
    setEdited(JSON.stringify(newProfile) !== JSON.stringify(profile))
  }, [newProfile, profile])

  const handleProfileChange = useCallback(
    (event: FormEvent) => {
      const element = event.target as HTMLInputElement
      const key: keyof ExtendedProfile = element.name as keyof ExtendedProfile
      const temp = { ...newProfile }
      temp[key] = element.value
      setNewProfile(temp)
      updateEdited()
      if (validateField(key, newProfile)) invalidFields[key] = false
    },
    [invalidFields, newProfile, updateEdited]
  )

  return (
    <I18nextProvider i18n={i18next}>
      <SC.Main id='account-profile' className='account-profile'>
        <SC.Paper style={{ boxShadow: containerBoxShadow }}>
          <SC.GridContainer style={{ rowGap: '2.5rem' }}>
            <SC.GridItemContainer>
              {!noPicture && (
                <SC.GridItemContainer>
                  <Picture
                    newProfile={newProfile}
                    updateEdited={updateEdited}
                    setNewProfile={setNewProfile}
                  />
                  <WarningText />
                </SC.GridItemContainer>
              )}
              <SC.GridItem>
                <ProtectedInput
                  inputText={{
                    id: 'email',
                    label: t('email'),
                    icon: emailIcon
                  }}
                  options={options}
                  newProfile={newProfile}
                />
              </SC.GridItem>
              <SC.GridItemMd6>
                <StandardInput
                  inputText={{
                    id: 'firstName',
                    label: t('firstName')
                  }}
                  options={options}
                  invalidFields={invalidFields}
                  newProfile={newProfile}
                  handleProfileChange={handleProfileChange}
                />
              </SC.GridItemMd6>
              <SC.GridItemMd6>
                <StandardInput
                  inputText={{
                    id: 'lastName',
                    label: t('lastName')
                  }}
                  options={options}
                  invalidFields={invalidFields}
                  newProfile={newProfile}
                  handleProfileChange={handleProfileChange}
                />
              </SC.GridItemMd6>
              <SC.GridItem>
                <StandardInput
                  inputText={{
                    id: 'organization',
                    label: t('organization'),
                    icon: organizationIcon
                  }}
                  options={options}
                  invalidFields={invalidFields}
                  newProfile={newProfile}
                  handleProfileChange={handleProfileChange}
                />
              </SC.GridItem>
              <SC.GridItem>
                <PhoneInput
                  inputText={{
                    id: 'phoneNumber',
                    label: t('phoneNumber'),
                    icon: phoneIcon
                  }}
                  options={options}
                  handleProfileChange={handleProfileChange}
                  invalidFields={invalidFields}
                  newProfile={newProfile}
                  onChangeNumber={onChangeNumber}
                />
              </SC.GridItem>
            </SC.GridItemContainer>
            {withOptData && (
              <>
                <SC.GridItem>
                  <SC.SectionLabel>{t('personalData')}</SC.SectionLabel>
                  <SC.GridContainer
                    style={{
                      justifyContent: 'space-between',
                      rowGap: '1.75rem'
                    }}
                  >
                    <SC.GridItemMd6>
                      <StandardInput
                        inputText={{
                          id: 'birthDate',
                          label: t('birthDate'),
                          type: 'date'
                        }}
                        options={options}
                        handleProfileChange={handleProfileChange}
                        invalidFields={invalidFields}
                        newProfile={newProfile}
                        notRequired
                      />
                    </SC.GridItemMd6>
                    <SC.GridItemMd6>
                      <StandardSelect
                        selectInput={{
                          id: 'gender',
                          label: t('gender')
                        }}
                        options={options}
                        notRequired
                        invalidFields={invalidFields}
                        newProfile={newProfile}
                        handleProfileChange={handleProfileChange}
                      />
                    </SC.GridItemMd6>
                  </SC.GridContainer>
                </SC.GridItem>
                <SC.GridItem>
                  <SC.SectionLabel>{t('background')}</SC.SectionLabel>
                  <SC.GridContainer
                    style={{
                      justifyContent: 'space-between',
                      rowGap: '1.75rem'
                    }}
                  >
                    <SC.GridItemMd4>
                      <StandardInput
                        inputText={{
                          id: 'education',
                          label: t('education')
                        }}
                        options={options}
                        handleProfileChange={handleProfileChange}
                        invalidFields={invalidFields}
                        newProfile={newProfile}
                        notRequired
                      />
                    </SC.GridItemMd4>
                    <SC.GridItemMd4>
                      <StandardInput
                        inputText={{
                          id: 'profession',
                          label: t('profession')
                        }}
                        options={options}
                        handleProfileChange={handleProfileChange}
                        invalidFields={invalidFields}
                        newProfile={newProfile}
                        notRequired
                      />
                    </SC.GridItemMd4>
                    <SC.GridItemMd4>
                      <StandardInput
                        inputText={{
                          id: 'industry',
                          label: t('industry')
                        }}
                        options={options}
                        handleProfileChange={handleProfileChange}
                        invalidFields={invalidFields}
                        newProfile={newProfile}
                        notRequired
                      />
                    </SC.GridItemMd4>
                  </SC.GridContainer>
                </SC.GridItem>
                <SC.GridItem>
                  <SC.SectionLabel>{t('socialMedia')}</SC.SectionLabel>
                  <SC.GridContainer
                    style={{
                      justifyContent: 'space-between',
                      rowGap: '1.75rem'
                    }}
                  >
                    <SC.GridItemMd4>
                      <StandardInput
                        inputText={{
                          id: 'instagram',
                          label: 'Instagram'
                        }}
                        options={options}
                        handleProfileChange={handleProfileChange}
                        invalidFields={invalidFields}
                        newProfile={newProfile}
                        notRequired
                      />
                    </SC.GridItemMd4>
                    <SC.GridItemMd4>
                      <StandardInput
                        inputText={{
                          id: 'youtube',
                          label: 'Youtube'
                        }}
                        options={options}
                        handleProfileChange={handleProfileChange}
                        invalidFields={invalidFields}
                        newProfile={newProfile}
                        notRequired
                      />
                    </SC.GridItemMd4>
                    <SC.GridItemMd4>
                      <StandardInput
                        inputText={{
                          id: 'tiktok',
                          label: 'Tiktok'
                        }}
                        options={options}
                        handleProfileChange={handleProfileChange}
                        invalidFields={invalidFields}
                        newProfile={newProfile}
                        notRequired
                      />
                    </SC.GridItemMd4>
                  </SC.GridContainer>
                </SC.GridItem>
              </>
            )}
            {canRequestAccountDeletion && (
              <DeleteAccount options={options} onDelete={onDelete} />
            )}
            <SC.GridItemAction>
              <DiscardButton
                edited={edited}
                options={options}
                initProfile={initProfile}
                setInvalidFields={setInvalidFields}
                setEdited={setEdited}
              />

              <UpdateButton
                edited={edited}
                options={options}
                invalidFields={invalidFields}
                setInvalidFields={setInvalidFields}
                newProfile={newProfile}
                onUpdate={onUpdate}
              />
            </SC.GridItemAction>
          </SC.GridContainer>
        </SC.Paper>
      </SC.Main>
    </I18nextProvider>
  )
}
