import styled from '@emotion/styled'
import { useMemo } from 'react'
import { InputIcon, InvalidMessage, Label, TextInputContainer } from '../styles'
import { useTranslation } from 'react-i18next'
import type { FormEvent } from 'react'
import type {
  AccountProfileOptions,
  ExtendedProfile,
  InvalidField,
  Profile
} from '../types'

type InputType = {
  id: keyof Profile
  label: string
  icon?: string
  type?: string
}

interface Props {
  selectInput: InputType
  notRequired: boolean | undefined
  invalidFields: InvalidField
  newProfile: ExtendedProfile
  handleProfileChange: (event: FormEvent) => void
  options: AccountProfileOptions
}

const StandardSelect: React.FC<Props> = ({
  selectInput: { id, label, icon },
  notRequired,
  invalidFields,
  newProfile,
  handleProfileChange,
  options: { mode }
}) => {
  const { t } = useTranslation('accountProfile')

  const genderOptions = useMemo(
    () => [
      {
        label: t('female'),
        value: 'female'
      },
      {
        label: t('male'),
        value: 'male'
      }
    ],
    [t]
  )

  return (
    <TextInputContainer>
      <Label htmlFor={id}>
        {!notRequired && <span style={{ color: 'var(--red)' }}>*</span>} {label}
      </Label>
      <SelectInput
        name={id}
        style={{
          borderColor: invalidFields[id] ? 'var(--red)' : '',
          backgroundImage:
            mode === 'dark'
              ? `url("data:image/svg+xml;utf8,<svg fill='white' height='24' viewBox='0 0 24 24' width='24' xmlns='http://www.w3.org/2000/svg'><path d='M7 10l5 5 5-5z'/><path d='M0 0h24v24H0z' fill='none'/></svg>")`
              : `url("data:image/svg+xml;utf8,<svg fill='black' height='24' viewBox='0 0 24 24' width='24' xmlns='http://www.w3.org/2000/svg'><path d='M7 10l5 5 5-5z'/><path d='M0 0h24v24H0z' fill='none'/></svg>")`
        }}
        value={newProfile[id]}
        required={!notRequired}
        onChange={(e) => handleProfileChange(e)}
      >
        <option value='' disabled hidden>
          -
        </option>
        {genderOptions.map((category) => (
          <option key={category.value} value={category.value}>
            {category.label}
          </option>
        ))}
      </SelectInput>
      {icon && <InputIcon src={icon} />}
      {invalidFields[id] && (
        <InvalidMessage>{label} is required</InvalidMessage>
      )}
    </TextInputContainer>
  )
}

const SelectInput = styled.select({
  flexBasis: '100%',
  minWidth: 0,
  fontSize: '1rem',
  border: 'var(--input-border)',
  borderRadius: '0.3125rem',
  padding: '0.875rem',
  color: 'var(--light-black)',
  backgroundColor: 'var(--white)',
  '&:disabled': {
    border: 0,
    borderBottom: 'var(--input-border)',
    borderRadius: 0,
    background: 'var(--white)'
  },
  appearance: 'none',
  backgroundRepeat: 'no-repeat',
  backgroundPositionX: 'calc(100% - 0.5rem)',
  backgroundPositionY: '0.75rem',
  cursor: 'pointer',
  backgroundImage: `url("data:image/svg+xml;utf8,<svg fill='black' height='24' viewBox='0 0 24 24' width='24' xmlns='http://www.w3.org/2000/svg'><path d='M7 10l5 5 5-5z'/><path d='M0 0h24v24H0z' fill='none'/></svg>")`
})

export default StandardSelect
