import * as React from 'react'
import * as SC from './styles'

import MediaQuery from 'react-responsive'
import i18n from '../../lib/i18n'
import PhoneInput from './component/PhoneInput'
import ChangePassword from './component/ChangePassword'
import SocmedInputDefined from './component/SocmedInputDefined'
import EditedFormInput from './component/EditedFormInput'

import instagramIcon from '../../assets/components/account/instagram.svg'
import youtubeIcon from '../../assets/components/account/youtube.svg'
import tiktokIcon from '../../assets/components/account/tiktok.svg'
import facebookIcon from '../../assets/components/account/facebook.svg'

import { useCallback, useEffect, useRef, useState } from 'react'
import { I18nextProvider, useTranslation } from 'react-i18next'
import { BiSolidEditAlt } from 'react-icons/bi'
import { useForm } from 'react-hook-form'

import type {
  AccountProfileOptions,
  AccountProfileProps,
  Profile
} from './types'

const defaultOptions: AccountProfileOptions = {
  language: 'id',
  mode: 'light',
  buttonColor: 'var(--fair-green)',
  canEditPhoneNumber: true
}

export const Account: React.FC<AccountProfileProps> = ({
  data,
  options,
  onUpdate,
  onChangeNumber,
  onChangePassword
}) => {
  options = { ...defaultOptions, ...options }

  const {
    t,
    i18n: { changeLanguage }
  } = useTranslation('account')

  const { language } = options

  useEffect(() => {
    changeLanguage(language)
  }, [changeLanguage, language])

  const [profile, setProfile] = useState<Profile>({
    email: '',
    firstName: '',
    lastName: '',
    organization: '',
    phoneNumber: '',
    instagram: '',
    youtube: '',
    tiktok: ''
  })

  const [newProfile, setNewProfile] = useState<Profile>({
    ...profile
  })

  const [isEditing, setIsEditing] = useState(false)
  const [isDisabled, setIsDisabled] = useState(true)
  const { register, setValue, reset, watch, handleSubmit } = useForm<Profile>()

  const formRef = useRef<HTMLFormElement | null>(null)

  const onSubmit = (data: Profile) => {
    onUpdate(data)
    setNewProfile(data)
  }

  const submit = () => {
    setValue('email', newProfile.email)
    setValue('phoneNumber', newProfile.phoneNumber)
    if (formRef.current) {
      formRef.current.dispatchEvent(
        new Event('submit', { cancelable: true, bubbles: true })
      )
    }
    setIsEditing(false)
  }

  const initProfile = useCallback(() => {
    setProfile(data)
    setNewProfile({
      ...data
    })
  }, [data])

  useEffect(() => {
    initProfile()
  }, [initProfile])

  useEffect(() => {
    const subscription = watch((_value, { name, type }) => {
      if (name && type) {
        setIsDisabled(false)
      } else {
        setIsDisabled(true)
      }
    })
    return () => subscription.unsubscribe()
  }, [watch])

  const handleEditClick = () => {
    setIsEditing(true)
    reset()
  }

  const handleCancel = () => {
    initProfile()
    setIsEditing(false)
  }

  return (
    <I18nextProvider i18n={i18n}>
      <SC.Main>
        <form ref={formRef} onSubmit={handleSubmit(onSubmit)}>
          <SC.Container>
            <SC.ProfileContainer>
              <SC.ProfileBox>
                <SC.FieldContainer>
                  <SC.FieldLabel>{t('email')}</SC.FieldLabel>
                  {isEditing ? (
                    <SC.FieldValue>
                      <SC.TextInput
                        type='email'
                        value={newProfile.email}
                        disabled
                        {...register('email')}
                      />
                    </SC.FieldValue>
                  ) : (
                    <SC.FieldValue>{newProfile.email}</SC.FieldValue>
                  )}
                </SC.FieldContainer>

                <EditedFormInput
                  name={'firstName'}
                  label={t('firstName')}
                  type='text'
                  value={newProfile.firstName}
                  isEdit={isEditing}
                  register={register('firstName')}
                />
                <EditedFormInput
                  name={'lastName'}
                  label={t('lastName')}
                  type='text'
                  value={newProfile.lastName}
                  isEdit={isEditing}
                  register={register('lastName')}
                />
                <EditedFormInput
                  name={'organization'}
                  label={t('organization')}
                  type='text'
                  value={newProfile.organization}
                  isEdit={isEditing}
                  register={register('organization')}
                />

                <PhoneInput
                  inputText={{
                    id: 'phoneNumber',
                    label: t('phoneNumber')
                  }}
                  isEdit={isEditing}
                  options={options}
                  register={register('phoneNumber')}
                  invalidFields={false}
                  newProfile={newProfile}
                  onChangeNumber={onChangeNumber}
                />

                <MediaQuery query='(min-width: 1147px)'>
                  <div style={{ padding: '16px 0px 16px 0px' }}>
                    <SC.Paragraph>
                      <SC.Text>{t('socialMedia')}</SC.Text>
                    </SC.Paragraph>
                  </div>

                  <EditedFormInput
                    name={'instagram'}
                    label={t('instagram')}
                    type='text'
                    value={newProfile.instagram}
                    isSocmed={true}
                    socmedIcon={instagramIcon}
                    isEdit={isEditing}
                    register={register('instagram')}
                  />
                  <EditedFormInput
                    name={'youtube'}
                    label={t('youtube')}
                    type='text'
                    value={newProfile.youtube}
                    isSocmed={true}
                    socmedIcon={youtubeIcon}
                    isEdit={isEditing}
                    register={register('youtube')}
                  />
                  <EditedFormInput
                    name={'tiktok'}
                    label={t('tiktok')}
                    type='text'
                    value={newProfile.tiktok}
                    isSocmed={true}
                    socmedIcon={tiktokIcon}
                    isEdit={isEditing}
                    register={register('tiktok')}
                  />

                  <SocmedInputDefined
                    name={'facebook'}
                    label={t('facebook')}
                    type='text'
                    value={newProfile.facebook}
                    socmedIcon={facebookIcon}
                    isEdit={isEditing}
                    register={register('facebook')}
                  />

                  {isEditing && (
                    <SC.ButtonContainer style={{ marginLeft: 'auto' }}>
                      <SC.CancelButton onClick={handleCancel}>
                        {t('cancel')}
                      </SC.CancelButton>
                      <SC.SaveButton
                        disabled={isDisabled}
                        type='button'
                        style={{ marginLeft: '64px' }}
                        onClick={submit}
                      >
                        {t('save')}
                      </SC.SaveButton>
                    </SC.ButtonContainer>
                  )}
                </MediaQuery>
              </SC.ProfileBox>
              {!isEditing && (
                <SC.ItemIcon>
                  <BiSolidEditAlt onClick={handleEditClick} />
                </SC.ItemIcon>
              )}
            </SC.ProfileContainer>
            <SC.ChangePasswordContainer>
              <SC.ChangePasswordBox>
                <div style={{ padding: '0px 0px 18px 0px' }}>
                  <SC.FieldLabel style={{ maxWidth: '100%' }}>
                    {t('changePassword')}
                  </SC.FieldLabel>
                </div>

                <SC.FieldValue style={{ flexBasis: '100%', maxWidth: '100%' }}>
                  <ChangePassword
                    options={options}
                    onChangePassword={onChangePassword}
                  />
                </SC.FieldValue>
              </SC.ChangePasswordBox>
            </SC.ChangePasswordContainer>
            <MediaQuery query='(max-width: 1146.98px)'>
              <SC.ProfileContainer>
                <SC.ProfileBox>
                  <div style={{ padding: '0px 0px 16px 0px' }}>
                    <SC.Paragraph>
                      <SC.Text>{t('socialMedia')}</SC.Text>
                    </SC.Paragraph>
                  </div>
                  <EditedFormInput
                    name={'instagram'}
                    label={t('instagram')}
                    type='text'
                    value={newProfile.instagram}
                    isSocmed={true}
                    socmedIcon={instagramIcon}
                    isEdit={isEditing}
                    register={register('instagram')}
                  />
                  <EditedFormInput
                    name={'youtube'}
                    label={t('youtube')}
                    type='text'
                    value={newProfile.youtube}
                    isSocmed={true}
                    socmedIcon={youtubeIcon}
                    isEdit={isEditing}
                    register={register('youtube')}
                  />
                  <EditedFormInput
                    name={'tiktok'}
                    label={t('tiktok')}
                    type='text'
                    value={newProfile.tiktok}
                    isSocmed={true}
                    socmedIcon={tiktokIcon}
                    isEdit={isEditing}
                    register={register('tiktok')}
                  />

                  <SocmedInputDefined
                    name={'facebook'}
                    label={t('facebook')}
                    type='text'
                    value={newProfile.facebook}
                    socmedIcon={facebookIcon}
                    isEdit={isEditing}
                    register={register('facebook')}
                  />
                </SC.ProfileBox>
              </SC.ProfileContainer>
              {isEditing && (
                <SC.ProfileContainer style={{ flexDirection: 'column' }}>
                  <SC.FieldValue style={{ marginLeft: 'auto' }}>
                    <SC.ButtonContainer>
                      <SC.CancelButton type='button' onClick={handleCancel}>
                        {t('cancel')}
                      </SC.CancelButton>
                      <SC.SaveButton
                        disabled={isDisabled}
                        type='button'
                        style={{ marginLeft: '64px' }}
                        onClick={submit}
                      >
                        {t('save')}
                      </SC.SaveButton>
                    </SC.ButtonContainer>
                  </SC.FieldValue>
                </SC.ProfileContainer>
              )}
            </MediaQuery>
          </SC.Container>
        </form>
      </SC.Main>
    </I18nextProvider>
  )
}
