import * as SC from '../styles'

import { useState, useEffect, useRef, FormEvent } from 'react'
import { useTranslation } from 'react-i18next'
import { useLocalStorage, useCountdown } from 'usehooks-ts'
import { timeRemaining, formatSecondsNoHour } from '../helper'

import type { AccountProfileOptions } from '../types'

interface Props {
  options: AccountProfileOptions
  onChangePassword?: () => Promise<void>
}

const ChangePassword: React.FC<Props> = ({
  options: { buttonColor },
  onChangePassword
}) => {
  const localStorageKey: string = `password::countdownChangePassword`

  const { t } = useTranslation('account')

  const [disableButton, setDisableButton] = useState(false)
  const [changePasswordRequested, setChangePasswordRequested] = useState(false)
  const [changePasswordSuccess, setChangePasswordSuccess] = useState(false)
  const [timeStamp, setTimeStamp] = useLocalStorage<string>(localStorageKey, '')
  const [countdownStart, setCountdownStart] = useState(timeRemaining(timeStamp))
  const [count, { startCountdown, resetCountdown }] = useCountdown({
    countStart: countdownStart,
    countStop: 0,
    intervalMs: 1000
  })

  const dialogRef = useRef(null)

  const handleShowModal = () => {
    if (dialogRef.current) {
      const element = dialogRef.current as HTMLDialogElement
      element.showModal()
    }
  }

  const handleCloseModal = () => {
    if (dialogRef.current) {
      const element = dialogRef.current as HTMLDialogElement
      element.close()
    }
    setDisableButton(false)
    setChangePasswordRequested(false)
  }

  const handleChangePassword = async () => {
    setDisableButton(true)
    try {
      if (onChangePassword) await onChangePassword()
      setChangePasswordSuccess(true)
      resetCountdown()
      setTimeStamp(new Date().toString())
    } catch {
      setChangePasswordSuccess(false)
    } finally {
      setDisableButton(false)
      setChangePasswordRequested(true)
    }
  }

  useEffect(() => {
    if (timeRemaining(timeStamp)) {
      startCountdown()
    }
    setCountdownStart(300)
  }, [timeStamp, startCountdown, setCountdownStart])

  return (
    <>
      {count ? (
        <SC.SaveButton
          disabled={true}
          type='button'
          style={{ width: 'auto', cursor: 'wait' }}
          onClick={handleShowModal}
        >
          {t('changePassword')} {formatSecondsNoHour(count)}
        </SC.SaveButton>
      ) : (
        <SC.SaveButton
          style={{ width: 'auto' }}
          type='button'
          onClick={handleShowModal}
        >
          {t('changePassword')}
        </SC.SaveButton>
      )}
      <SC.DialogConfirmation ref={dialogRef} onClick={handleCloseModal}>
        <SC.DialogContent
          onClick={(e: FormEvent) => {
            e.stopPropagation()
          }}
        >
          <SC.DialogTitle>{t('changePasswordTitle')}</SC.DialogTitle>
          {changePasswordRequested ? (
            <SC.DialogResult>
              {changePasswordSuccess
                ? t('changePasswordSuccess')
                : t('changePasswordFailed')}
            </SC.DialogResult>
          ) : (
            <SC.DialogDescription>
              {t('changePasswordDescription')}
            </SC.DialogDescription>
          )}
          {changePasswordRequested ? (
            <SC.DialogActionSuccess>
              <SC.DialogActionButton
                style={{
                  width: '78px',
                  color: 'white',
                  backgroundColor: buttonColor,
                  border: `1px solid ${buttonColor}`
                }}
                type='button'
                onClick={handleCloseModal}
              >
                Ok
              </SC.DialogActionButton>
            </SC.DialogActionSuccess>
          ) : (
            <SC.DialogAction
              style={{ width: '90%', margin: 'auto', marginTop: '40px' }}
            >
              <SC.DialogActionButton
                disabled={disableButton}
                style={{
                  width: '106px',
                  color: buttonColor,
                  backgroundColor: 'var(--white)',
                  border: `1px solid ${buttonColor}`
                }}
                type='button'
                onClick={handleCloseModal}
              >
                {t('changePasswordCancel')}
              </SC.DialogActionButton>
              <SC.DialogActionButton
                disabled={disableButton}
                type='button'
                style={{
                  width: '106px',
                  color: 'white',
                  backgroundColor: buttonColor,
                  border: `1px solid ${buttonColor}`
                }}
                onClick={handleChangePassword}
              >
                {t('changePasswordConfirm')}
              </SC.DialogActionButton>
            </SC.DialogAction>
          )}
        </SC.DialogContent>
      </SC.DialogConfirmation>
    </>
  )
}

export default ChangePassword
