import * as SC from './styles'
import i18next from '../../lib/i18n'

import { I18nextProvider, useTranslation } from 'react-i18next'
import { FormEvent, useState, useEffect, useCallback, useRef } from 'react'

import type { ContactFormOptions, ContactFormProps } from './types'

const emailRegex =
  /^(([^<>()[\]\\.,;:\s@"]+(\.[^<>()[\]\\.,;:\s@"]+)*)|(".+"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/
const phoneRegex =
  /^\(?(?:\+62|62|0)[ .-]?(?:[1-9])(?:\d{1,2})\)?[ .-]?\d{2,4}[ .-]?\d{2,4}[ .-]?\d{2,4}$/
const nameRegex =
  /^(([A-Za-z]+[-']?)*([A-Za-z]+)>?[.]?\s?)+([A-Za-z]+[-']?)*([A-Za-z]+)>?[.]?$/

const defaultOptions: ContactFormOptions = {
  background: 'transparent',
  mode: 'Light',
  language: 'id',
  categoryOptions: [],
  applicationName: '',
  sourceOptions: [],
  recaptcha: true,
  customTextColor: '#00b38f',
  customButtonColor: '#14bca2',
  shadowColor: '#00dd851a',
  disableMesageTrim: false
}

export const ContactForm: React.FC<ContactFormProps> = ({
  children,
  options,
  onSend
}) => {
  options = { ...defaultOptions, ...options }

  const {
    t,
    i18n: { changeLanguage }
  } = useTranslation('contactForm')

  const {
    background,
    mode,
    language,
    categoryOptions,
    applicationName,
    sourceOptions,
    recaptcha,
    customTextColor,
    customButtonColor,
    shadowColor,
    disableMesageTrim
  } = options

  useEffect(() => {
    changeLanguage(language)
  }, [changeLanguage, language])

  const isCategoryExists = categoryOptions && categoryOptions.length > 0
  const isSourceExists =
    sourceOptions && sourceOptions.length > 0 && applicationName !== ''

  const [state, setState] = useState({
    name: '',
    email: '',
    phone: '',
    category: '',
    message: '',
    source: '',
    other: '',
    isInvalidName: false,
    isInvalidEmail: false,
    isInvalidPhone: false,
    isInvalidCategory: false,
    isInvalidMessage: false,
    isInvalidSource: false,
    isInvalidOther: false
  })

  const [isOtherSource, setIsOtherSource] = useState(false)
  const formRef = useRef<HTMLDivElement | null>(null)

  const resetForm = useCallback(() => {
    setState({
      name: '',
      email: '',
      phone: '',
      category: '',
      message: '',
      source: '',
      other: '',
      isInvalidName: false,
      isInvalidEmail: false,
      isInvalidPhone: false,
      isInvalidCategory: false,
      isInvalidMessage: false,
      isInvalidSource: false,
      isInvalidOther: false
    })
    setIsOtherSource(false)
  }, [])

  const handleOnSubmitErrorEvent = useCallback(
    (event: Event) => {
      const customEvent = event as CustomEvent
      const eventData = customEvent.detail?.data
      if (!eventData) {
        resetForm()
      }
    },
    [resetForm]
  )

  useEffect(() => {
    const element = formRef.current

    const eventListener = (event: Event) => {
      handleOnSubmitErrorEvent(event)
    }

    if (element) {
      element.addEventListener(
        'on-submit-contact-form-error',
        eventListener as EventListener
      )
    }

    return () => {
      if (element) {
        element.removeEventListener(
          'on-submit-contact-form-error',
          eventListener as EventListener
        )
      }
    }
  }, [handleOnSubmitErrorEvent])

  const handleChangeName = (e: FormEvent<HTMLInputElement>) => {
    const name = e.currentTarget.value.trim()
    if (name && nameRegex.test(name)) {
      setState({ ...state, name, isInvalidName: false })
    } else {
      setState({ ...state, name, isInvalidName: true })
    }
  }

  const handleChangeEmail = (e: FormEvent<HTMLInputElement>) => {
    const email = e.currentTarget.value.trim()
    if (email && emailRegex.test(email)) {
      setState({ ...state, email, isInvalidEmail: false })
    } else {
      setState({ ...state, email, isInvalidEmail: true })
    }
  }

  const handleChangePhone = (e: FormEvent<HTMLInputElement>) => {
    const phone = e.currentTarget.value.trim()
    if (phone && phoneRegex.test(phone)) {
      setState({ ...state, phone, isInvalidPhone: false })
    } else {
      setState({ ...state, phone, isInvalidPhone: true })
    }
  }

  const handleChangeCategory = (e: FormEvent<HTMLSelectElement>) => {
    const category = e.currentTarget.value.trim()
    if (category || !isCategoryExists) {
      setState({ ...state, category, isInvalidCategory: false })
    } else {
      setState({ ...state, category, isInvalidCategory: true })
    }
  }

  const handleChangeMessage = (e: FormEvent<HTMLTextAreaElement>) => {
    const message = disableMesageTrim
      ? e.currentTarget.value
      : e.currentTarget.value.trim()
    if (message) {
      setState({ ...state, message, isInvalidMessage: false })
    } else {
      setState({ ...state, message, isInvalidMessage: true })
    }
  }

  const handleChangeSource = (e: FormEvent<HTMLSelectElement>) => {
    const source = e.currentTarget.value.trim()
    if (source || !isSourceExists) {
      setState({ ...state, source, isInvalidSource: false })
    } else {
      setState({ ...state, source, isInvalidSource: true })
    }
    if (
      source.toLocaleLowerCase() === 'lainnya' ||
      source.toLocaleLowerCase() === 'other'
    ) {
      setIsOtherSource(true)
    } else {
      setIsOtherSource(false)
    }
  }

  const handleChangeOther = (e: FormEvent<HTMLInputElement>) => {
    const other = e.currentTarget.value.trim()
    if (other) {
      setState({ ...state, other, isInvalidOther: false })
    } else {
      setState({ ...state, other, isInvalidOther: true })
    }
  }

  const handleSubmit = (e: FormEvent<HTMLFormElement>) => {
    e.preventDefault()
    const isInvalidName = !(state.name && nameRegex.test(state.name))
    const isInvalidEmail = !(state.email && emailRegex.test(state.email))
    const isInvalidPhone = !(state.phone && phoneRegex.test(state.phone))
    const isInvalidCategory = !(state.category || !isCategoryExists)
    const isInvalidMessage = !state.message
    const isInvalidSource = !(state.source || !isSourceExists)
    const isInvalidOther = isOtherSource && !state.other
    setState({
      ...state,
      isInvalidName,
      isInvalidEmail,
      isInvalidPhone,
      isInvalidCategory,
      isInvalidMessage,
      isInvalidSource,
      isInvalidOther
    })
    const success = !(
      isInvalidName ||
      isInvalidEmail ||
      isInvalidPhone ||
      isInvalidCategory ||
      isInvalidMessage ||
      isInvalidSource ||
      isInvalidOther
    )
    if (success) {
      let source = state.source
      if (
        source.toLowerCase() == 'other' ||
        source.toLowerCase() == 'lainnya'
      ) {
        source = state.other
      }
      const data = {
        name: state.name,
        email: state.email,
        phone: state.phone,
        category: state.category,
        message: state.message,
        source: source
      }
      onSend(data)
    }
  }

  return (
    <I18nextProvider i18n={i18next}>
      <SC.ContactForm
        ref={formRef}
        className='contact-form-component'
        style={{
          background: background
            ? background
            : mode === 'Dark'
              ? 'black'
              : 'white'
        }}
      >
        <SC.Row>
          <SC.ColLeft>
            <div>
              <SC.Title
                style={{
                  color: mode === 'Dark' ? 'white' : '#231f20'
                }}
              >
                {t('textTitle')}{' '}
                <span style={{ color: customTextColor }}>
                  {t('textHighlight')}
                </span>
              </SC.Title>
              <SC.Description
                style={{
                  color: mode === 'Dark' ? 'white' : '231f20'
                }}
              >
                {t('textDescription')}
              </SC.Description>
            </div>
            {children}
          </SC.ColLeft>
          <SC.ColGap />
          <SC.ColRight>
            <SC.Panel
              autoCorrect='off'
              autoCapitalize='off'
              autoComplete='off'
              spellCheck='false'
              style={{
                boxShadow: `0px 30px 50px 0px ${
                  shadowColor ? shadowColor : '#00dd851a'
                }`,
                backgroundColor: mode === 'Dark' ? 'black' : 'white',
                border: mode === 'Dark' ? '1px solid #00dd85' : 'none'
              }}
              onSubmit={handleSubmit}
            >
              <SC.Group>
                <div>
                  <SC.Label htmlFor='name'>{t('textName')}</SC.Label>
                </div>
                <div>
                  <SC.Field
                    type='text'
                    id='name'
                    name='name'
                    value={state.name}
                    placeholder={t('textName')}
                    style={{
                      backgroundColor: mode === 'Dark' ? 'black' : ' white',
                      color: mode === 'Dark' ? 'white' : 'black',
                      border: state.isInvalidName
                        ? '1px solid #ff0000'
                        : '1px solid #dfdfdf'
                    }}
                    onChange={handleChangeName}
                  />
                </div>
                {state.isInvalidName && (
                  <div>
                    <SC.AlertMessage>{t('alertName')}</SC.AlertMessage>
                  </div>
                )}
              </SC.Group>
              <SC.Group>
                <div>
                  <SC.Label htmlFor='email'>Email</SC.Label>
                </div>
                <div>
                  <SC.Field
                    type='text'
                    id='email'
                    name='email'
                    value={state.email}
                    placeholder='Email'
                    style={{
                      backgroundColor: mode === 'Dark' ? 'black' : ' white',
                      color: mode === 'Dark' ? 'white' : 'black',
                      border: state.isInvalidEmail
                        ? '1px solid #ff0000'
                        : '1px solid #dfdfdf'
                    }}
                    onChange={handleChangeEmail}
                  />
                </div>
                {state.isInvalidEmail && (
                  <div>
                    <SC.AlertMessage>{t('alertEmail')}</SC.AlertMessage>
                  </div>
                )}
              </SC.Group>
              <SC.Group>
                <div>
                  <SC.Label htmlFor='phone'>{t('textPhone')}</SC.Label>
                </div>
                <div>
                  <SC.Field
                    type='text'
                    id='phone'
                    name='phone'
                    value={state.phone}
                    placeholder={t('textPhone')}
                    style={{
                      backgroundColor: mode === 'Dark' ? 'black' : ' white',
                      color: mode === 'Dark' ? 'white' : 'black',
                      border: state.isInvalidPhone
                        ? '1px solid #ff0000'
                        : '1px solid #dfdfdf'
                    }}
                    onChange={handleChangePhone}
                  />
                </div>
                {state.isInvalidPhone && (
                  <div>
                    <SC.AlertMessage>{t('alertPhone')}</SC.AlertMessage>
                  </div>
                )}
              </SC.Group>
              {isCategoryExists && (
                <SC.Group>
                  <div>
                    <SC.Label htmlFor='category'>{t('textCategory')}</SC.Label>
                  </div>
                  <div>
                    <SC.Select
                      id='category'
                      name='category'
                      value={state.category}
                      style={{
                        backgroundColor: mode === 'Dark' ? 'black' : ' white',
                        color: mode === 'Dark' ? 'white' : 'black',
                        backgroundImage:
                          mode === 'Dark'
                            ? `url("data:image/svg+xml;utf8,<svg fill='white' height='24' viewBox='0 0 24 24' width='24' xmlns='http://www.w3.org/2000/svg'><path d='M7 10l5 5 5-5z'/><path d='M0 0h24v24H0z' fill='none'/></svg>")`
                            : `url("data:image/svg+xml;utf8,<svg fill='black' height='24' viewBox='0 0 24 24' width='24' xmlns='http://www.w3.org/2000/svg'><path d='M7 10l5 5 5-5z'/><path d='M0 0h24v24H0z' fill='none'/></svg>")`,
                        border: state.isInvalidCategory
                          ? '1px solid #ff0000'
                          : '1px solid #dfdfdf'
                      }}
                      onChange={handleChangeCategory}
                    >
                      <option value=''>{t('textCategory')}</option>
                      {language &&
                        categoryOptions.map((category) => (
                          <option key={category.value} value={category.value}>
                            {category[language]}
                          </option>
                        ))}
                    </SC.Select>
                  </div>
                </SC.Group>
              )}
              {state.isInvalidCategory && (
                <div>
                  <SC.AlertMessage>{t('alertCategory')}</SC.AlertMessage>
                </div>
              )}
              <SC.Group>
                <div>
                  <SC.Label htmlFor='message'>{t('textMessage')}</SC.Label>
                </div>
                <div>
                  <SC.TextArea
                    id='message'
                    name='message'
                    value={state.message}
                    placeholder={t('textMessage')}
                    style={{
                      backgroundColor: mode === 'Dark' ? 'black' : ' white',
                      color: mode === 'Dark' ? 'white' : 'black',
                      border: state.isInvalidMessage
                        ? '1px solid #ff0000'
                        : '1px solid #dfdfdf'
                    }}
                    onChange={handleChangeMessage}
                  />
                </div>
                {state.isInvalidMessage && (
                  <div>
                    <SC.AlertMessage>{t('alertMessage')}</SC.AlertMessage>
                  </div>
                )}
              </SC.Group>
              {isSourceExists && (
                <SC.Group>
                  <div>
                    <SC.Label htmlFor='source'>
                      {t('textInfo', { application_name: applicationName })}
                    </SC.Label>
                  </div>
                  <div>
                    <SC.Select
                      id='source'
                      name='source'
                      value={state.source}
                      style={{
                        backgroundColor: mode === 'Dark' ? 'black' : ' white',
                        color: mode === 'Dark' ? 'white' : 'black',
                        backgroundImage:
                          mode === 'Dark'
                            ? `url("data:image/svg+xml;utf8,<svg fill='white' height='24' viewBox='0 0 24 24' width='24' xmlns='http://www.w3.org/2000/svg'><path d='M7 10l5 5 5-5z'/><path d='M0 0h24v24H0z' fill='none'/></svg>")`
                            : `url("data:image/svg+xml;utf8,<svg fill='black' height='24' viewBox='0 0 24 24' width='24' xmlns='http://www.w3.org/2000/svg'><path d='M7 10l5 5 5-5z'/><path d='M0 0h24v24H0z' fill='none'/></svg>")`,
                        border: state.isInvalidSource
                          ? '1px solid #ff0000'
                          : '1px solid #dfdfdf'
                      }}
                      onChange={handleChangeSource}
                    >
                      <option value=''>{t('textSource')}</option>
                      {language &&
                        sourceOptions.map((source) => (
                          <option key={source.value} value={source.value}>
                            {source[language]}
                          </option>
                        ))}
                    </SC.Select>
                  </div>
                  {state.isInvalidSource && (
                    <div>
                      <SC.AlertMessage>{t('alertSource')}</SC.AlertMessage>
                    </div>
                  )}
                  {isOtherSource && (
                    <>
                      <div>
                        <SC.Field
                          type='text'
                          id='other'
                          name='other'
                          value={state.other}
                          placeholder={t('textWrite')}
                          style={{
                            backgroundColor:
                              mode === 'Dark' ? 'black' : ' white',
                            color: mode === 'Dark' ? 'white' : 'black',
                            border: state.isInvalidOther
                              ? '1px solid #ff0000'
                              : '1px solid #dfdfdf'
                          }}
                          onChange={handleChangeOther}
                        />
                      </div>
                      {state.isInvalidOther && (
                        <div>
                          <SC.AlertMessage>{t('alertOther')}</SC.AlertMessage>
                        </div>
                      )}
                    </>
                  )}
                </SC.Group>
              )}
              {recaptcha && (
                <div>
                  <SC.Recaptcha>
                    This site is protected by reCAPTCHA and the Google{' '}
                    <a
                      href='https://policies.google.com/privacy'
                      rel='noopener noreferrer'
                      target='_blank'
                      style={{ color: customTextColor }}
                    >
                      Privacy Policy
                    </a>{' '}
                    and{' '}
                    <a
                      href='https://policies.google.com/terms'
                      rel='noopener noreferrer'
                      target='_blank'
                      style={{ color: customTextColor }}
                    >
                      Terms of Service
                    </a>{' '}
                    apply.
                  </SC.Recaptcha>
                </div>
              )}
              <SC.Group>
                <div>
                  <SC.Button
                    type='submit'
                    id='send'
                    style={{
                      backgroundColor: customButtonColor
                    }}
                  >
                    {t('textSend')}
                  </SC.Button>
                </div>
              </SC.Group>
            </SC.Panel>
          </SC.ColRight>
        </SC.Row>
      </SC.ContactForm>
    </I18nextProvider>
  )
}
