import * as SC from './styles'
import i18next from '../../../lib/i18n'

import { I18nextProvider } from 'react-i18next'
import { useTranslation } from 'react-i18next'
import { FormEvent, useState, useEffect } from 'react'
import {
  errorCardNumber,
  errorCardHolder,
  errorCardExpiry,
  errorCardCvv,
  errorProfileName
} from './helper'

import FolderIcon from '../../../assets/components/payment-profile/folder.svg'
import CardNumberIcon from '../../../assets/components/payment-profile/card-number.svg'
import CardHolderIcon from '../../../assets/components/payment-profile/card-holder.svg'
import CalendarIcon from '../../../assets/components/payment-profile/calendar.svg'
import CvvIcon from '../../../assets/components/payment-profile/cvv.svg'
import WarningIcon from '../../../assets/components/payment-profile/warning.svg'
import { BankLogoMap, BinMap } from '../bin/bin'

import type {
  BillingualText,
  PaymentProfileProps,
  PaymentProfileOptions
} from './types'

const defaultOptions: PaymentProfileOptions = {
  language: 'id',
  submitText: {
    id: 'Tambah',
    en: 'Add'
  },
  cancelText: {
    id: 'Hapus',
    en: 'Discard'
  },
  termsUrl: '',
  actionColor: '#56ab2f',
  disableColor: '#d0ecb4',
  boxShadow: '0px 15px 30px -5px rgba(86, 171, 47, 0.15)',
  disableCancel: true,
  disableSubmit: true,
  hideCvv: false
}

export const PaymentProfile: React.FC<PaymentProfileProps> = ({
  data,
  options,
  onAdd,
  onCancel
}) => {
  options = { ...defaultOptions, ...options }

  const {
    t,
    i18n: { changeLanguage }
  } = useTranslation('paymentProfile')

  const {
    profileName,
    cardNumber,
    cardHolder,
    cardExpiry,
    cardCvv,
    isTermsAgree
  } = data

  let { submitText, cancelText } = options
  const {
    language,
    termsUrl,
    actionColor,
    disableColor,
    boxShadow,
    disableCancel,
    disableSubmit,
    hideCvv
  } = options

  useEffect(() => {
    changeLanguage(language)
  }, [changeLanguage, language])

  const [stateProfileName, setStateProfileName] = useState(profileName)
  const [stateCardNumber, setStateCardNumber] = useState(cardNumber)
  const [stateCardHolder, setStateCardHolder] = useState(cardHolder)
  const [stateCardExpiry, setStateCardExpiry] = useState(cardExpiry)
  const [stateCardCvv, setStateCardCvv] = useState(cardCvv)
  const [stateIsTermsAgree, setStateIsTermsAgree] = useState(isTermsAgree)
  const [stateDisableCancel, setStateDisableCancel] = useState(disableCancel)
  const [stateDisableSubmit, setStateDisableSubmit] = useState(disableSubmit)

  const [invalidProfileName, setInvalidProfileName] = useState<BillingualText>({
    id: '',
    en: ''
  })
  const [invalidCardNumber, setInvalidCardNumber] = useState<BillingualText>({
    id: '',
    en: ''
  })
  const [invalidCardHolder, setInvalidCardHolder] = useState<BillingualText>({
    id: '',
    en: ''
  })
  const [invalidCardExpiry, setInvalidCardExpiry] = useState<BillingualText>({
    id: '',
    en: ''
  })
  const [invalidCardCvv, setInvalidCardCvv] = useState<BillingualText>({
    id: '',
    en: ''
  })

  const [cardLogo, setCardLogo] = useState(CardNumberIcon)

  if (typeof submitText === 'string') {
    submitText = JSON.parse(submitText)
  }
  if (typeof cancelText === 'string') {
    cancelText = JSON.parse(cancelText)
  }

  const _changeButtons = () => {
    setStateDisableSubmit(!stateIsTermsAgree)
    setStateDisableCancel(!stateIsTermsAgree)
  }

  const _handleChangeCardNumber = () => {
    const cardNumberLength = stateCardNumber.trim().length

    if (cardNumberLength >= 6) {
      const binDigits = stateCardNumber.trim().replace(/-/g, '').slice(0, 6)
      const bank = BinMap[Number(binDigits)]
      if (bank) {
        setCardLogo(BankLogoMap[bank])
      } else {
        setCardLogo(CardNumberIcon)
      }
    }

    if (cardNumberLength === 4) {
      setStateCardNumber(stateCardNumber + '-')
    }
    if (cardNumberLength === 9) {
      setStateCardNumber(stateCardNumber + '-')
    }
    if (cardNumberLength === 14) {
      setStateCardNumber(stateCardNumber + '-')
    }
  }

  const _handleChangeCardExpiry = () => {
    if (
      stateCardExpiry.trim().length === 2 &&
      !stateCardExpiry.trim().includes('/')
    ) {
      setStateCardExpiry(stateCardExpiry + '/')
    }
  }

  const _dispatchSubmit = () => {
    if (!stateDisableSubmit) {
      const profileName = stateProfileName.trim()
      const cardNumber = stateCardNumber.trim()
      const cardHolder = stateCardHolder.trim()
      const cardExpiry = stateCardExpiry.trim()
      const cardCvv = stateCardCvv.trim() || ''
      const isTermsAgree = stateIsTermsAgree

      const _errorProfileName = errorProfileName(profileName)
      const _errorCardNumber = errorCardNumber(cardNumber)
      const _errorCardHolder = errorCardHolder(cardHolder)
      const _errorCardExpiry = errorCardExpiry(cardExpiry)
      const _errorCardCVV = errorCardCvv(cardCvv)

      setInvalidProfileName(_errorProfileName)
      setInvalidCardNumber(_errorCardNumber)
      setInvalidCardHolder(_errorCardHolder)
      setInvalidCardExpiry(_errorCardExpiry)
      setInvalidCardCvv(_errorCardCVV)
      if (
        language &&
        _errorProfileName[language] === '' &&
        _errorCardNumber[language] === '' &&
        _errorCardHolder[language] === '' &&
        _errorCardExpiry[language] === '' &&
        (_errorCardCVV[language] === '' || hideCvv) &&
        isTermsAgree
      ) {
        const data = {
          profileName,
          cardNumber,
          cardHolder,
          cardExpiry,
          cardCvv,
          isTermsAgree
        }
        onAdd(data)
      }
    }
  }

  const _dispatchCancel = () => {
    if (!stateDisableCancel) {
      setStateProfileName('')
      setStateCardNumber('')
      setStateCardHolder('')
      setStateCardExpiry('')
      setStateCardCvv('')
      setStateIsTermsAgree(false)
      setStateDisableSubmit(true)
      setStateDisableCancel(true)

      onCancel()
    }
  }

  return (
    <I18nextProvider i18n={i18next}>
      <SC.PaymentRoot style={{ boxShadow: boxShadow }}>
        <SC.SectionForm>
          <SC.PaymentTitle>{t('title')}</SC.PaymentTitle>
          <SC.IndicatesRequired>
            <SC.MandatorySign>* </SC.MandatorySign>
            {t('requiredText')}
          </SC.IndicatesRequired>
        </SC.SectionForm>
        <form
          autoCorrect='off'
          autoCapitalize='off'
          autoComplete='off'
          spellCheck='false'
          onSubmit={_dispatchSubmit}
        >
          <SC.SectionForm>
            <SC.PmMb8>
              <SC.MandatorySign>* </SC.MandatorySign>
              {t('profileNameText')}
            </SC.PmMb8>
            <SC.PaymentFieldInput>
              <SC.IconInput src={FolderIcon} alt='icon-profile-name' />
              <SC.InputField
                id='profile-name'
                type='text'
                placeholder={t('profileNamePlaceholder')}
                value={stateProfileName}
                onChange={(e: FormEvent<HTMLInputElement>) =>
                  setStateProfileName(e.currentTarget.value)
                }
              />
            </SC.PaymentFieldInput>
            <SC.FormValidation>
              {language && invalidProfileName[language].length > 0 && (
                <>
                  <img src={WarningIcon} alt='icon-warning' />
                  <SC.PmMl8>{invalidProfileName[language]}</SC.PmMl8>
                </>
              )}
            </SC.FormValidation>
          </SC.SectionForm>

          <SC.SectionForm>
            <SC.PmMb8>
              <SC.MandatorySign>* </SC.MandatorySign>
              {t('cardNumberText')}
            </SC.PmMb8>
            <SC.PaymentFieldInput>
              {cardLogo === CardNumberIcon ? (
                <SC.IconInput src={cardLogo} alt='card-number-icon' />
              ) : (
                <SC.BankLogo src={cardLogo} alt='card-number-icon' />
              )}
              <SC.InputField
                id='card-number'
                value={stateCardNumber}
                onChange={(e: FormEvent<HTMLInputElement>) =>
                  setStateCardNumber(e.currentTarget.value)
                }
                type='text'
                placeholder='xxxx-xxxx-xxxx-xxxx'
                minLength={13}
                maxLength={19}
                onKeyUp={_handleChangeCardNumber}
                onKeyDown={_handleChangeCardNumber}
              />
            </SC.PaymentFieldInput>
            <SC.FormValidation>
              {language && invalidCardNumber[language].length > 0 && (
                <>
                  <img src={WarningIcon} alt='icon-warning' />
                  <SC.PmMl8>{invalidCardNumber[language]}</SC.PmMl8>
                </>
              )}
            </SC.FormValidation>
          </SC.SectionForm>

          <SC.SectionForm>
            <SC.PmMb8>
              <SC.MandatorySign>* </SC.MandatorySign>
              {t('cardHolderText')}
            </SC.PmMb8>
            <SC.PaymentFieldInput>
              <SC.IconInput src={CardHolderIcon} alt='icon-card-holder' />
              <SC.InputField
                id='card-holder'
                value={stateCardHolder}
                onChange={(e: FormEvent<HTMLInputElement>) =>
                  setStateCardHolder(e.currentTarget.value)
                }
                type='text'
                placeholder={t('cardNamePlaceholder')}
              />
            </SC.PaymentFieldInput>
            <SC.FormValidation>
              {language && invalidCardHolder[language].length > 0 && (
                <>
                  <img src={WarningIcon} alt='icon-warning' />
                  <SC.PmMl8>{invalidCardHolder[language]}</SC.PmMl8>
                </>
              )}
            </SC.FormValidation>
          </SC.SectionForm>

          <SC.Row>
            {hideCvv ? (
              <SC.SectionForm
                style={{
                  marginRight: hideCvv ? '0' : '2%',
                  marginBottom: hideCvv ? '0' : ''
                }}
              >
                <SC.PmMb8>
                  <SC.MandatorySign>* </SC.MandatorySign>
                  {t('expiryText')}
                </SC.PmMb8>
                <SC.PaymentFieldInput>
                  <SC.IconInput src={CalendarIcon} alt='icon-card-holder' />
                  <SC.InputField
                    id='card-expiry'
                    value={stateCardExpiry}
                    onChange={(e: FormEvent<HTMLInputElement>) =>
                      setStateCardExpiry(e.currentTarget.value)
                    }
                    type='text'
                    placeholder='MM/YY'
                    minLength={5}
                    maxLength={5}
                    onKeyUp={_handleChangeCardExpiry}
                    onKeyDown={_handleChangeCardExpiry}
                  />
                </SC.PaymentFieldInput>
                <SC.FormValidation>
                  {language && invalidCardExpiry[language].length > 0 && (
                    <>
                      <img src={WarningIcon} alt='icon-warning' />
                      <SC.PmMl8>{invalidCardExpiry[language]}</SC.PmMl8>
                    </>
                  )}
                </SC.FormValidation>
              </SC.SectionForm>
            ) : (
              <SC.Column
                style={{
                  marginRight: hideCvv ? '0' : '2%',
                  marginBottom: hideCvv ? '0' : ''
                }}
              >
                <SC.PmMb8>
                  <SC.MandatorySign>* </SC.MandatorySign>
                  {t('expiryText')}
                </SC.PmMb8>
                <SC.PaymentFieldInput>
                  <SC.IconInput src={CalendarIcon} alt='icon-card-holder' />
                  <SC.InputField
                    id='card-expiry'
                    value={stateCardExpiry}
                    onChange={(e: FormEvent<HTMLInputElement>) =>
                      setStateCardExpiry(e.currentTarget.value)
                    }
                    type='text'
                    placeholder='MM/YY'
                    minLength={5}
                    maxLength={5}
                    onKeyUp={_handleChangeCardExpiry}
                    onKeyDown={_handleChangeCardExpiry}
                  />
                </SC.PaymentFieldInput>
                <SC.FormValidation>
                  {language && invalidCardExpiry[language].length > 0 && (
                    <>
                      <img src={WarningIcon} alt='icon-warning' />
                      <SC.PmMl8>{invalidCardExpiry[language]}</SC.PmMl8>
                    </>
                  )}
                </SC.FormValidation>
              </SC.Column>
            )}
            <SC.Column
              style={{ marginLeft: '2%', display: hideCvv ? 'none' : '' }}
            >
              <SC.PmMb8>
                <SC.MandatorySign>* </SC.MandatorySign>CVV
              </SC.PmMb8>
              <SC.PaymentFieldInput>
                <SC.IconInput src={CvvIcon} alt='icon-cvv' />
                <SC.InputField
                  id='card-cvv'
                  value={stateCardCvv}
                  onChange={(e: FormEvent<HTMLInputElement>) =>
                    setStateCardCvv(e.currentTarget.value)
                  }
                  type='text'
                  placeholder='CVV'
                  minLength={3}
                  maxLength={3}
                />
              </SC.PaymentFieldInput>
              <SC.FormValidation>
                {language && invalidCardCvv[language].length > 0 && (
                  <>
                    <img src={WarningIcon} alt='icon-warning' />
                    <SC.PmMl8>{invalidCardCvv[language]}</SC.PmMl8>
                  </>
                )}
              </SC.FormValidation>
            </SC.Column>
          </SC.Row>

          <SC.SectionTerms>
            <SC.PmMb4>{t('conditionsText')}</SC.PmMb4>
            <SC.SectionTermsCheck>
              {t('checkTermsText')}{' '}
              <SC.TextTerms>
                <a href={termsUrl} style={{ color: actionColor }}>
                  {t('termsText')}
                </a>
              </SC.TextTerms>
              <input
                id='is-terms-agree'
                checked={stateIsTermsAgree}
                type='checkbox'
                onInput={_changeButtons}
                onChange={() => setStateIsTermsAgree(!stateIsTermsAgree)}
              />
              <SC.CheckMark />
            </SC.SectionTermsCheck>
          </SC.SectionTerms>
        </form>
        <SC.PaymentAction>
          {stateDisableCancel ? (
            <SC.ButtonCancelInactive
              style={{
                border: stateDisableCancel
                  ? '1px solid ' + disableColor
                  : '1px solid ' + actionColor,
                color: stateDisableCancel ? disableColor : actionColor
              }}
              onClick={_dispatchCancel}
            >
              {language && (cancelText as BillingualText)[language]}
            </SC.ButtonCancelInactive>
          ) : (
            <SC.ButtonCancelActive
              style={{
                border: stateDisableCancel
                  ? '1px solid ' + disableColor
                  : '1px solid ' + actionColor,
                color: stateDisableCancel ? disableColor : actionColor
              }}
              onClick={_dispatchCancel}
            >
              {language && (cancelText as BillingualText)[language]}
            </SC.ButtonCancelActive>
          )}
          {stateDisableSubmit ? (
            <SC.ButtonSubmitInactive
              style={{
                background: stateDisableSubmit ? disableColor : actionColor
              }}
              onClick={_dispatchSubmit}
            >
              {language && (submitText as BillingualText)[language]}
            </SC.ButtonSubmitInactive>
          ) : (
            <SC.ButtonSubmitActive
              style={{
                background: stateDisableSubmit ? disableColor : actionColor
              }}
              onClick={_dispatchSubmit}
            >
              {language && (submitText as BillingualText)[language]}
            </SC.ButtonSubmitActive>
          )}
        </SC.PaymentAction>
      </SC.PaymentRoot>
    </I18nextProvider>
  )
}

export default PaymentProfile
