import * as React from 'react'
import * as SC from './styles'

import DatePicker from 'react-datepicker'
import 'react-datepicker/dist/react-datepicker.css'
import i18n from '../../lib/i18n'

import Search from '../../assets/components/transaction-history/search.svg'
import Calendar from '../../assets/components/transaction-history/calendar.svg'
import IllustrattionImage from '../../assets/components/transaction-history/illustration-image.svg'
import LeftActive from '../../assets/components/transaction-history/left-black.svg'
import LeftPassive from '../../assets/components/transaction-history/left-grey.svg'
import RightActive from '../../assets/components/transaction-history/right-black.svg'
import RightPassive from '../../assets/components/transaction-history/right-grey.svg'
import Watch from '../../assets/components/transaction-history/watch.svg'
import Download from '../../assets/components/transaction-history/download.svg'

import { I18nextProvider, useTranslation } from 'react-i18next'
import {
  FormEvent,
  useState,
  useEffect,
  useMemo,
  useCallback,
  forwardRef
} from 'react'
import { stringDateTime, stringPrice } from './helper'
import { useDebounceValue } from 'usehooks-ts'

import type {
  Transactions,
  Parameter,
  TransactionHistoryOptions,
  TransactionHistoryProps
} from './types'

const defaultOptions: TransactionHistoryOptions = {
  display: 'List',
  seeDetailsColor: '#56ab2f',
  successColor: '#16ca5e',
  waitingForPaymentColor: '#ffb800',
  failedColor: '#f34949',
  language: 'id',
  mode: 'Light',
  boxShadow: '0px 15px 30px -5px rgba(86, 171, 47, 0.15)',
  noFilter: false,
  showListBtn: false,
  enableInvoiceDownload: false
}

export const TransactionHistory: React.FC<TransactionHistoryProps> = ({
  data,
  parameter,
  options,
  onChangeParameter,
  onDownloadInvoice
}) => {
  options = { ...defaultOptions, ...options }

  const {
    t,
    i18n: { changeLanguage }
  } = useTranslation('transactionHistory')

  const { transactions, total } = data

  const { page, pageSize, search, start, end, status, planType } = parameter

  const {
    display,
    seeDetailsColor,
    successColor,
    waitingForPaymentColor,
    failedColor,
    language,
    mode,
    boxShadow,
    noFilter,
    showListBtn,
    enableInvoiceDownload
  } = options

  useEffect(() => {
    changeLanguage(language)
  }, [changeLanguage, language])

  const [indexDetails, setIndexDetails] = useState(0)
  const [textMessage, setTextMessage] = useState('')
  const [textShow, setTextShow] = useState('')
  const [pagination, setPagination] = useState<string[]>([])
  const [isLeftPage, setIsLeftPage] = useState(true)
  const [isRightPage, setIsRightPage] = useState(true)

  const [statePage, setStatePage] = useState(page)
  const [statePageSize, setStatePageSize] = useState(pageSize)
  const [stateSearch, setStateSearch] = useState(search)
  const [debouncedValue, setValue] = useDebounceValue(stateSearch, 500)
  const [stateStart, setStateStart] = useState<Date | undefined>(start)
  const [stateEnd, setStateEnd] = useState<Date | undefined>(end)
  const [stateStatus, setStateStatus] = useState(status)
  const [statePlanType, setStatePlanType] = useState(planType)
  const [stateDisplay, setStateDisplay] = useState(display)

  const CustomInputLight = forwardRef<HTMLDivElement>(
    (data: { value?: string; onClick?: () => void }, ref) => (
      <SC.FilterDateInnerLight ref={ref} onClick={data.onClick}>
        <SC.CalendarIcon src={Calendar} />
        <SC.DateInput>
          {data.value ? data.value : t('textFilterByDate')}
        </SC.DateInput>
      </SC.FilterDateInnerLight>
    )
  )

  const CustomInputDark = forwardRef<HTMLDivElement>(
    (data: { value?: string; onClick?: () => void }, ref) => (
      <SC.FilterDateInnerDark ref={ref} onClick={data.onClick}>
        <SC.CalendarIcon src={Calendar} />
        <SC.DateInput>
          {data.value ? data.value : t('textFilterByDate')}
        </SC.DateInput>
      </SC.FilterDateInnerDark>
    )
  )

  const items = useMemo(() => {
    if (transactions) {
      if (typeof transactions === 'string') {
        return JSON.parse(transactions)
      } else {
        return transactions
      }
    }
  }, [transactions])

  const isFilterChanged = useMemo(() => {
    return (
      stateSearch !== '' ||
      stateStart !== null ||
      stateEnd !== null ||
      stateStatus !== 'Status' ||
      statePlanType !== 'Plan type'
    )
  }, [stateSearch, stateStart, stateEnd, stateStatus, statePlanType])

  const params: Parameter = useMemo(
    () => ({
      page: page,
      pageSize: pageSize,
      search: search,
      start: start,
      end: end,
      status: status,
      planType: planType
    }),
    [page, pageSize, search, start, end, status, planType]
  )

  const refreshItems = useCallback(
    (newParameter: Parameter) => {
      onChangeParameter(newParameter)
    },
    [onChangeParameter]
  )

  useEffect(() => {
    if (debouncedValue !== search) {
      refreshItems({ ...params, search: debouncedValue })
    }
  }, [refreshItems, params, debouncedValue, search])

  const handleSearch = (e: FormEvent<HTMLInputElement>) => {
    setStateSearch(e.currentTarget.value)
    setValue(e.currentTarget.value)
  }

  const handleChangeStatus = (e: FormEvent<HTMLSelectElement>) => {
    setStateStatus(e.currentTarget.value)
    refreshItems({ ...params, status: e.currentTarget.value })
  }

  const handleChangePlanType = (e: FormEvent<HTMLSelectElement>) => {
    setStatePlanType(e.currentTarget.value)
    refreshItems({ ...params, planType: e.currentTarget.value })
  }

  const onChangeDate = (dates: [Date | null, Date | null]) => {
    const [start, end] = dates
    setStateStart(start ?? undefined)
    setStateEnd(end ?? undefined)
    if (start && end) {
      refreshItems({ ...params, start: start, end: end })
    }
  }

  const resetFilter = () => {
    setStateSearch('')
    setValue('')
    setStateStart(undefined)
    setStateEnd(undefined)
    setStateStatus('Status')
    setStatePlanType('Plan type')
    refreshItems({
      page: 1,
      pageSize: 'All',
      search: '',
      start: undefined,
      end: undefined,
      status: 'Status',
      planType: 'Plan type'
    })
  }

  const handleChangePageSize = (e: FormEvent<HTMLSelectElement>) => {
    setStatePageSize(e.currentTarget.value)
    refreshItems({ ...params, pageSize: e.currentTarget.value })
  }

  const seeDetails = (index: number) => {
    setIndexDetails(index)
    setStateDisplay('Details')
  }

  const seeList = () => {
    setStateDisplay('List')
  }

  const goToPage = (page: number) => {
    setStatePage(page)
    refreshItems({ ...params, page: page })
  }

  const handleInvoice = () => {
    onDownloadInvoice(items[indexDetails].orderId)
  }

  useEffect(() => {
    const refreshPagination = () => {
      const pageSize =
        statePageSize === 'All' ? Number(total) : Number(statePageSize)
      const totalPage = Math.ceil(Number(total) / pageSize)
      const paginationTemp = []
      if (totalPage <= 7) {
        for (let x = 1; x <= totalPage; x++) {
          paginationTemp.push(x.toString())
        }
      } else {
        paginationTemp.push('1')
        if (statePage <= 4) {
          for (let x = 2; x <= 5; x++) {
            paginationTemp.push(x.toString())
          }
          paginationTemp.push('...')
          paginationTemp.push(totalPage.toString())
        } else if (totalPage - statePage > 3) {
          paginationTemp.push('...')
          paginationTemp.push((statePage - 1).toString())
          paginationTemp.push(statePage.toString())
          paginationTemp.push((statePage + 1).toString())
          paginationTemp.push('...')
          paginationTemp.push(totalPage.toString())
        } else if (totalPage - statePage <= 3) {
          paginationTemp.push('...')
          for (let x = totalPage - 4; x <= totalPage; x++) {
            paginationTemp.push(x.toString())
          }
        }
      }
      setPagination(paginationTemp)
      setIsLeftPage(Number(statePage) !== 1)
      setIsRightPage(Number(statePage) !== totalPage)
    }

    if (Number(total)) {
      if (Number(total) === 1) {
        setTextShow(
          `${t('textShowing')} 1 ${t('textOf')} 1 ${t('textTransaction')}`
        )
      } else if (statePageSize === 'All') {
        setTextShow(
          `${t('textShowing')} 1 - ${total} ${t('textOf')} ${total} ${t(
            'textTransactions'
          )}`
        )
      } else if (total <= statePage * Number(statePageSize)) {
        setTextShow(
          `${t('textShowing')} ${
            (statePage - 1) * Number(statePageSize) + 1
          } - ${total} ${t('textOf')} ${total} ${t('textTransactions')}`
        )
      } else {
        setTextShow(
          `${t('textShowing')} ${
            (statePage - 1) * Number(statePageSize) + 1
          } - ${statePage * Number(statePageSize)} ${t('textOf')} ${total} ${t(
            'textTransactions'
          )}`
        )
      }
    } else {
      setTextShow(`${t('textShowing')} 0 ${t('textTransactions')}`)
      if (statePlanType === 'Single purchase') {
        setTextMessage('No single purchase transactions')
      } else if (statePlanType === 'Subscription') {
        setTextMessage('No subscription transactions')
      } else {
        setTextMessage('No transactions')
      }
    }
    refreshPagination()
  }, [statePage, statePageSize, statePlanType, total, t])

  return (
    <I18nextProvider i18n={i18n}>
      <SC.TransactionHistory
        style={{
          backgroundColor: 'transparent'
        }}
      >
        <div style={{ display: stateDisplay === 'List' ? 'block' : 'none' }}>
          {!noFilter && (
            <>
              <SC.Header
                style={{
                  backgroundColor: mode === 'Dark' ? '#252e36' : '#f6f6f6',
                  borderColor: mode === 'Dark' ? '#212422' : '#dfdfdf'
                }}
              >
                <SC.SearchBox
                  style={{
                    backgroundColor: mode === 'Dark' ? '#3d4a46' : 'white',
                    borderColor: mode === 'Dark' ? '#2ec998' : '#dfdfdf'
                  }}
                >
                  {mode === 'Dark' ? (
                    <SC.SearchInputDark
                      id='search'
                      type='text'
                      placeholder={t('textSearchPlaceholder')}
                      style={{
                        color: 'white'
                      }}
                      value={stateSearch}
                      onChange={handleSearch}
                    />
                  ) : (
                    <SC.SearchInput
                      id='search'
                      type='text'
                      placeholder={t('textSearchPlaceholder')}
                      style={{
                        color: '#464646'
                      }}
                      value={stateSearch}
                      onChange={handleSearch}
                    />
                  )}
                  <SC.SearchIcon src={Search} alt='Search' />
                </SC.SearchBox>
                {mode === 'Dark' ? (
                  <SC.FilterDateDark>
                    <DatePicker
                      customInput={<CustomInputDark />}
                      dateFormat='dd/MM/yyyy'
                      onChange={onChangeDate}
                      startDate={stateStart}
                      endDate={stateEnd}
                      selectsRange
                      showMonthDropdown
                      showYearDropdown
                      dropdownMode='select'
                      locale={language}
                    />
                  </SC.FilterDateDark>
                ) : (
                  <SC.FilterDateLight>
                    <DatePicker
                      customInput={<CustomInputLight />}
                      dateFormat='dd/MM/yyyy'
                      onChange={onChangeDate}
                      startDate={stateStart}
                      endDate={stateEnd}
                      selectsRange
                      showMonthDropdown
                      showYearDropdown
                      dropdownMode='select'
                      locale={language}
                    />
                  </SC.FilterDateLight>
                )}
              </SC.Header>
              <SC.FilterBar>
                <SC.Section>
                  <SC.FilterBy
                    style={{
                      color: mode === 'Dark' ? 'white' : '#a3a3a3'
                    }}
                  >
                    {t('textFilterBy')}
                  </SC.FilterBy>
                  <div>
                    <SC.Select
                      id='status'
                      style={{
                        width: '13.5rem',

                        color: mode === 'Dark' ? 'white' : '#464646',
                        backgroundImage:
                          mode === 'Dark'
                            ? `url("data:image/svg+xml;utf8,<svg fill='white' height='24' viewBox='0 0 24 24' width='24' xmlns='http://www.w3.org/2000/svg'><path d='M7 10l5 5 5-5z'/><path d='M0 0h24v24H0z' fill='none'/></svg>")`
                            : `url("data:image/svg+xml;utf8,<svg fill='black' height='24' viewBox='0 0 24 24' width='24' xmlns='http://www.w3.org/2000/svg'><path d='M7 10l5 5 5-5z'/><path d='M0 0h24v24H0z' fill='none'/></svg>")`
                      }}
                      value={stateStatus}
                      onChange={handleChangeStatus}
                    >
                      <option
                        style={{
                          backgroundColor: mode === 'Dark' ? '#3d4a46' : 'white'
                        }}
                      >
                        {t('textStatus')}
                      </option>
                      <option
                        style={{
                          backgroundColor: mode === 'Dark' ? '#3d4a46' : 'white'
                        }}
                        value='success'
                      >
                        {t('textSuccess')}
                      </option>
                      <option
                        style={{
                          backgroundColor: mode === 'Dark' ? '#3d4a46' : 'white'
                        }}
                        value='pending'
                      >
                        {t('textWaitingPayment')}
                      </option>
                      <option
                        style={{
                          backgroundColor: mode === 'Dark' ? '#3d4a46' : 'white'
                        }}
                        value='failed'
                      >
                        {t('textFailed')}
                      </option>
                    </SC.Select>
                  </div>
                  <div>
                    <SC.Select
                      id='plan-type'
                      style={{
                        width: '10rem',

                        color: mode === 'Dark' ? 'white' : '#464646',
                        backgroundImage:
                          mode === 'Dark'
                            ? `url("data:image/svg+xml;utf8,<svg fill='white' height='24' viewBox='0 0 24 24' width='24' xmlns='http://www.w3.org/2000/svg'><path d='M7 10l5 5 5-5z'/><path d='M0 0h24v24H0z' fill='none'/></svg>")`
                            : `url("data:image/svg+xml;utf8,<svg fill='black' height='24' viewBox='0 0 24 24' width='24' xmlns='http://www.w3.org/2000/svg'><path d='M7 10l5 5 5-5z'/><path d='M0 0h24v24H0z' fill='none'/></svg>")`
                      }}
                      value={statePlanType}
                      onChange={handleChangePlanType}
                    >
                      <option
                        style={{
                          backgroundColor: mode === 'Dark' ? '#3d4a46' : 'white'
                        }}
                        value='Plan type'
                      >
                        {t('textPlanType')}
                      </option>
                      <option
                        style={{
                          backgroundColor: mode === 'Dark' ? '#3d4a46' : 'white'
                        }}
                        value='Single purchase'
                      >
                        {t('textSinglePurchase')}
                      </option>
                      <option
                        style={{
                          backgroundColor: mode === 'Dark' ? '#3d4a46' : 'white'
                        }}
                        value='Subscription'
                      >
                        {t('textSubscription')}
                      </option>
                    </SC.Select>
                  </div>
                </SC.Section>
                <SC.Section>
                  {isFilterChanged ? (
                    <SC.ResetFilterActive onClick={resetFilter}>
                      {t('textResetFilter')}
                    </SC.ResetFilterActive>
                  ) : (
                    <SC.ResetFilterPassive onClick={resetFilter}>
                      {t('textResetFilter')}
                    </SC.ResetFilterPassive>
                  )}
                </SC.Section>
              </SC.FilterBar>
            </>
          )}
          <SC.StatusBar>
            <SC.Section>
              <SC.Items
                style={{ color: mode === 'Dark' ? 'white' : '#a3a3a3' }}
              >
                {textShow}
              </SC.Items>
            </SC.Section>
            <SC.Section>
              <SC.Items
                style={{ color: mode === 'Dark' ? 'white' : '#a3a3a3' }}
              >
                {t('textItemsPerPage')}
              </SC.Items>
              <div>
                <SC.Select
                  id='page-size'
                  style={{
                    width: '7rem',

                    color: mode === 'Dark' ? 'white' : '#464646',
                    backgroundImage:
                      mode === 'Dark'
                        ? `url("data:image/svg+xml;utf8,<svg fill='white' height='24' viewBox='0 0 24 24' width='24' xmlns='http://www.w3.org/2000/svg'><path d='M7 10l5 5 5-5z'/><path d='M0 0h24v24H0z' fill='none'/></svg>")`
                        : `url("data:image/svg+xml;utf8,<svg fill='black' height='24' viewBox='0 0 24 24' width='24' xmlns='http://www.w3.org/2000/svg'><path d='M7 10l5 5 5-5z'/><path d='M0 0h24v24H0z' fill='none'/></svg>")`
                  }}
                  value={statePageSize}
                  onChange={handleChangePageSize}
                >
                  <option
                    style={{
                      backgroundColor: mode === 'Dark' ? '#3d4a46' : 'white'
                    }}
                  >
                    5
                  </option>
                  <option
                    style={{
                      backgroundColor: mode === 'Dark' ? '#3d4a46' : 'white'
                    }}
                  >
                    10
                  </option>
                  <option
                    style={{
                      backgroundColor: mode === 'Dark' ? '#3d4a46' : 'white'
                    }}
                  >
                    20
                  </option>
                  <option
                    style={{
                      backgroundColor: mode === 'Dark' ? '#3d4a46' : 'white'
                    }}
                  >
                    50
                  </option>
                  <option
                    style={{
                      backgroundColor: mode === 'Dark' ? '#3d4a46' : 'white'
                    }}
                  >
                    100
                  </option>
                  <option
                    style={{
                      backgroundColor: mode === 'Dark' ? '#3d4a46' : 'white'
                    }}
                    value='All'
                  >
                    {t('textAll')}
                  </option>
                </SC.Select>
              </div>
            </SC.Section>
          </SC.StatusBar>
          {items.length ? (
            <SC.WithTransactions>
              {items.map((item: Transactions, index: number) => (
                <SC.Card
                  key={item.orderId}
                  style={{
                    boxShadow: boxShadow,
                    backgroundColor: mode === 'Dark' ? '#1e8262' : 'white'
                  }}
                >
                  <SC.CardHeader>
                    <SC.DateTime
                      style={{
                        color: mode === 'Dark' ? '#4dffc6' : '#7a7a7a'
                      }}
                    >
                      {language && stringDateTime(item.dateTime, language)}
                    </SC.DateTime>
                    <SC.Container>
                      {item.status === 'success' ? (
                        <SC.StatusSuccess style={{ color: successColor }}>
                          {t('textSuccess')}
                        </SC.StatusSuccess>
                      ) : item.status === 'failed' ? (
                        <SC.StatusFailed style={{ color: failedColor }}>
                          {t('textFailed')}
                        </SC.StatusFailed>
                      ) : (
                        <SC.StatusWaiting
                          style={{ color: waitingForPaymentColor }}
                        >
                          {t('textWaitingPayment')}
                        </SC.StatusWaiting>
                      )}
                    </SC.Container>
                  </SC.CardHeader>
                  <SC.CardBody>
                    <div>
                      <SC.PlanName
                        style={{ color: mode === 'Dark' ? 'white' : '#464646' }}
                      >
                        {item.planName}
                      </SC.PlanName>
                      <SC.CardSection>
                        <SC.CardLabel
                          style={{
                            color: mode === 'Dark' ? '#27d9a0' : '#7a7a7a'
                          }}
                        >
                          Order ID
                        </SC.CardLabel>
                        <SC.OrderId
                          style={{
                            color: mode === 'Dark' ? 'white' : '#464646'
                          }}
                        >
                          {item.orderId}
                        </SC.OrderId>
                      </SC.CardSection>
                      <SC.CardSection>
                        <SC.CardLabel
                          style={{
                            color: mode === 'Dark' ? '#27d9a0' : '#7a7a7a'
                          }}
                        >
                          {t('textPlanType')}
                        </SC.CardLabel>
                        <SC.PlanType
                          style={{
                            color: mode === 'Dark' ? 'white' : '#464646'
                          }}
                        >
                          {item.planType}
                        </SC.PlanType>
                      </SC.CardSection>
                    </div>
                    <SC.CardRight>
                      <SC.TotalPrice
                        style={{
                          color: mode === 'Dark' ? 'white' : '#464646'
                        }}
                      >
                        {stringPrice(item.totalPrice)}
                      </SC.TotalPrice>
                      <SC.SeeDetails
                        style={{ color: seeDetailsColor }}
                        onClick={() => seeDetails(index)}
                      >
                        {t('textSeeDetails')}
                      </SC.SeeDetails>
                    </SC.CardRight>
                  </SC.CardBody>
                </SC.Card>
              ))}
              <SC.Pagination>
                {isLeftPage ? (
                  <SC.ArrowActive
                    style={{
                      backgroundColor: mode === 'Dark' ? 'black' : 'white'
                    }}
                    onClick={() => goToPage(page - 1)}
                  >
                    <img src={mode === 'Dark' ? LeftPassive : LeftActive} />
                  </SC.ArrowActive>
                ) : (
                  <SC.ArrowPassive>
                    <img src={mode === 'Dark' ? LeftActive : LeftPassive} />
                  </SC.ArrowPassive>
                )}
                {pagination.map((item, index) => (
                  <span key={index}>
                    {item === '...' ? (
                      <SC.PageNumberEllipsis>...</SC.PageNumberEllipsis>
                    ) : (
                      <>
                        {item === page.toString() ? (
                          <SC.PageNumberActive
                            style={{
                              color: mode === 'Dark' ? 'white' : '#464646'
                            }}
                          >
                            {page}
                          </SC.PageNumberActive>
                        ) : (
                          <SC.PageNumberPassive
                            onClick={() => goToPage(Number(item))}
                          >
                            {item}
                          </SC.PageNumberPassive>
                        )}
                      </>
                    )}
                  </span>
                ))}
                {isRightPage ? (
                  <SC.ArrowActive
                    style={{
                      backgroundColor: mode === 'Dark' ? 'black' : 'white'
                    }}
                    onClick={() => goToPage(page + 1)}
                  >
                    <img src={mode === 'Dark' ? RightPassive : RightActive} />
                  </SC.ArrowActive>
                ) : (
                  <SC.ArrowPassive>
                    <img src={mode === 'Dark' ? RightActive : RightPassive} />
                  </SC.ArrowPassive>
                )}
              </SC.Pagination>
            </SC.WithTransactions>
          ) : (
            <SC.NoTransactions>
              <SC.IllustrationBox>
                <div>
                  <img src={IllustrattionImage} />
                </div>
              </SC.IllustrationBox>
              <SC.IllustrationText>
                <SC.IllustrationTitle
                  style={{ color: mode === 'Dark' ? 'white' : '#464646' }}
                >
                  {textMessage}
                </SC.IllustrationTitle>
                <SC.IllustrationDescription
                  style={{
                    color: mode === 'Dark' ? 'white' : '#7a7a7a'
                  }}
                >
                  {t('textIllustrationDescription')}
                </SC.IllustrationDescription>
              </SC.IllustrationText>
            </SC.NoTransactions>
          )}
        </div>

        {stateDisplay === 'Details' && indexDetails in items && (
          <div>
            {showListBtn && (
              <div style={{ marginBottom: '1rem', boxShadow: boxShadow }}>
                <SC.DetailsButton style={{ width: '100%' }} onClick={seeList}>
                  &#8592; {t('textAllHistory')}
                </SC.DetailsButton>
              </div>
            )}

            {(items[indexDetails].status === 'new' ||
              items[indexDetails].status === 'pending') && (
              <SC.DetailsWarning style={{ boxShadow: boxShadow }}>
                <SC.DetailsWatch>
                  <img src={Watch} />
                  {t('textClickContinue')}
                </SC.DetailsWatch>
                <SC.DetailsPayment
                  href={items[indexDetails].paymentLink}
                  target='midtrans-payment-gateway'
                >
                  {t('textMakePayment')}
                </SC.DetailsPayment>
              </SC.DetailsWarning>
            )}

            <SC.DetailsCard
              style={{
                boxShadow: boxShadow,
                backgroundColor: mode === 'Dark' ? '#1e8262' : 'white'
              }}
            >
              <SC.DetailsHeader>
                <SC.DetailsTitle
                  style={{ color: mode === 'Dark' ? 'white' : '#464646' }}
                >
                  {t('textTransaction').charAt(0).toUpperCase() +
                    t('textTransaction').slice(1)}
                </SC.DetailsTitle>
                {items[indexDetails].status === 'success' &&
                  enableInvoiceDownload === true && (
                    <SC.DetailsButton onClick={handleInvoice}>
                      <img src={Download} />
                      {t('textDownloadInvoice')}
                    </SC.DetailsButton>
                  )}
              </SC.DetailsHeader>
              <SC.DetailsSeparator />
              <SC.DetailsSection>
                <SC.DetailsLabel
                  style={{ color: mode === 'Dark' ? '#27d9a0' : '#464646' }}
                >
                  Order ID
                </SC.DetailsLabel>
                <SC.DetailsValue
                  style={{
                    color: mode === 'Dark' ? 'white' : '#464646'
                  }}
                >
                  {items[indexDetails].orderId}
                </SC.DetailsValue>
              </SC.DetailsSection>
              <SC.DetailsSection>
                <SC.DetailsLabel
                  style={{ color: mode === 'Dark' ? '#27d9a0' : '#464646' }}
                >
                  Status
                </SC.DetailsLabel>
                <SC.DetailsValue
                  style={{
                    color: mode === 'Dark' ? 'white' : '#464646'
                  }}
                >
                  {items[indexDetails].status === 'success'
                    ? t('textSuccess')
                    : items[indexDetails].status === 'failed'
                      ? t('textFailed')
                      : t('textWaitingPayment')}
                </SC.DetailsValue>
              </SC.DetailsSection>
              {items[indexDetails].status === 'failed' && (
                <SC.DetailsSection>
                  <SC.DetailsLabel
                    style={{ color: mode === 'Dark' ? '#27d9a0' : '#464646' }}
                  >
                    {t('textReasonForCancel')}
                  </SC.DetailsLabel>
                  <SC.DetailsValue
                    style={{
                      color: mode === 'Dark' ? 'white' : '#464646'
                    }}
                  >
                    {t('textPaymentDlExpired')}
                  </SC.DetailsValue>
                </SC.DetailsSection>
              )}
              <SC.DetailsSection>
                <SC.DetailsLabel
                  style={{ color: mode === 'Dark' ? '#27d9a0' : '#464646' }}
                >
                  {t('textDateTime')}
                </SC.DetailsLabel>
                <SC.DetailsValue
                  style={{
                    color: mode === 'Dark' ? 'white' : '#464646'
                  }}
                >
                  {language &&
                    stringDateTime(items[indexDetails].dateTime, language)}
                </SC.DetailsValue>
              </SC.DetailsSection>
            </SC.DetailsCard>
            <SC.DetailsCard
              style={{
                boxShadow: boxShadow,
                backgroundColor: mode === 'Dark' ? '#1e8262' : 'white'
              }}
            >
              <SC.DetailsTitle
                style={{ color: mode === 'Dark' ? 'white' : '#464646' }}
              >
                {t('textProductDetails')}
              </SC.DetailsTitle>
              <SC.DetailsSeparator />
              <SC.DetailsSection>
                <SC.DetailsLabel
                  style={{ color: mode === 'Dark' ? '#27d9a0' : '#464646' }}
                >
                  {t('textPlanType')}
                </SC.DetailsLabel>
                <SC.DetailsValue
                  style={{
                    color: mode === 'Dark' ? 'white' : '#464646'
                  }}
                >
                  {items[indexDetails].planType}
                </SC.DetailsValue>
              </SC.DetailsSection>
              <SC.DetailsSection>
                <SC.DetailsLabel
                  style={{ color: mode === 'Dark' ? '#27d9a0' : '#464646' }}
                >
                  {t('textPlanName')}
                </SC.DetailsLabel>
                <SC.DetailsValue
                  style={{
                    color: mode === 'Dark' ? 'white' : '#464646'
                  }}
                >
                  {items[indexDetails].planName}
                </SC.DetailsValue>
              </SC.DetailsSection>
              <SC.DetailsSection>
                <SC.DetailsLabel
                  style={{ color: mode === 'Dark' ? '#27d9a0' : '#464646' }}
                >
                  {t('textFeatures')}
                </SC.DetailsLabel>
                <SC.DetailsValue
                  style={{
                    color: mode === 'Dark' ? 'white' : '#464646'
                  }}
                >
                  <SC.DetailsList>
                    {items[indexDetails].features.map(
                      (item: string, index: number) => (
                        <li key={index}>{item}</li>
                      )
                    )}
                  </SC.DetailsList>
                </SC.DetailsValue>
              </SC.DetailsSection>
              <SC.DetailsTitle
                style={{ color: mode === 'Dark' ? 'white' : '#464646' }}
              >
                {t('textPaymentDetails')}
              </SC.DetailsTitle>
              <SC.DetailsSeparator />
              <SC.DetailsSection>
                <SC.DetailsLabel
                  style={{ color: mode === 'Dark' ? '#27d9a0' : '#464646' }}
                >
                  {t('textPaymentMethod')}
                </SC.DetailsLabel>
                <SC.DetailsValue
                  style={{
                    color: mode === 'Dark' ? 'white' : '#464646'
                  }}
                >
                  {items[indexDetails].paymentMethod}
                </SC.DetailsValue>
              </SC.DetailsSection>
              <SC.DetailsSection>
                <SC.DetailsLabel
                  style={{ color: mode === 'Dark' ? '#27d9a0' : '#464646' }}
                >
                  {t('textPackageName')}
                  <SC.DetailsValueSpan
                    style={{
                      color: mode === 'Dark' ? 'white' : '#464646'
                    }}
                  >
                    {' '}
                    (1 {t('textProduct')})
                  </SC.DetailsValueSpan>
                </SC.DetailsLabel>
                <SC.DetailsValueSpan
                  style={{
                    color: mode === 'Dark' ? 'white' : '#464646'
                  }}
                >
                  {items[indexDetails].planName}
                </SC.DetailsValueSpan>
              </SC.DetailsSection>
              <SC.DetailsSection>
                <SC.DetailsLabel
                  style={{ color: mode === 'Dark' ? '#27d9a0' : '#464646' }}
                >
                  {t('textGrandTotal')}
                </SC.DetailsLabel>
                <SC.DetailsTotal
                  style={{
                    color: mode === 'Dark' ? 'white' : '#56ab2f'
                  }}
                >
                  {stringPrice(items[indexDetails].totalPrice)}
                </SC.DetailsTotal>
              </SC.DetailsSection>
            </SC.DetailsCard>
          </div>
        )}
      </SC.TransactionHistory>
    </I18nextProvider>
  )
}

export default TransactionHistory
